@@ -1,18 +1,19 @@
 from __future__ import generator_stop
 
+import concurrent.futures
 import logging
 import signal
 import threading
+from collections import deque
+from concurrent.futures._base import FINISHED
 from multiprocessing import Pool
 from queue import Empty, Full, Queue
-from typing import TYPE_CHECKING
+from typing import Any, Callable, Generic, Iterable, Iterator, Optional, Sequence, Set, Tuple, TypeVar, Union
 
 from .exceptions import NoResult, assert_choice
 
-if TYPE_CHECKING:
-	from typing import Any, Callable, Iterable, Iterator, Optional, Sequence, Tuple, TypeVar, Union
-	T = TypeVar("T")
-	U = TypeVar("U")
+T = TypeVar("T")
+U = TypeVar("U")
 
 logger = logging.getLogger(__name__)
 
@@ -355,7 +356,7 @@ class ProgressThreadPool(object):
 			w.daemon = True  # program will end even if threads are still running
 			w.start()
 
-	def start(self, callable, *args, **kwargs):
+	def start(self, callable, *args, **kwargs):  # todo: add optional task id here.
 		self.waiting_queue.put((callable, args, kwargs))
 
 	def finalize(self, result):
@@ -392,7 +393,7 @@ class ProgressThreadPool(object):
 	def get_running(self):  # no locks, inconsistent data
 		return list(task for task in (w.get() for w in self.workers) if task)
 
-class BoundedQueue(object):
+class BoundedQueue:
 
 	# similar: pip install bounded-iterator
 
@@ -432,16 +433,86 @@ class BoundedQueue(object):
 
 		self.semaphore.release()
 
+class BufferedIterable(Generic[T]):
+
+	def __init__(self, it: Iterable[T], bufsize: int):
+		self.iterable = it
+		self.iterator = None
+		self.buffer = deque([])
+		self.bufsize = bufsize
+
+	def __iter__(self) -> Iterator[T]:
+
+		self.iterator = iter(self.iterable)
+		return self
+
+	def __next__(self) -> T:
+
+		if self.iterator is None:
+			raise TypeError
+
+		try:
+			while len(self.buffer) < self.bufsize:
+				item = next(self.iterator)
+				self.buffer.append(item)
+		except StopIteration:
+			pass
+
+		try:
+			return self.buffer.popleft()
+		except IndexError:
+			raise StopIteration
+
+class CompletedFutures:
+
+	def __init__(self, it: Iterable[concurrent.futures.Future], bufsize: int, timeout=None):
+		self.iterable = it
+		self.iterator: Optional[Iterator[concurrent.futures.Future]] = None
+		self.futures: Set[concurrent.futures.Future] = set()
+		self.bufsize = bufsize
+		self.timeout = timeout
+
+	def __iter__(self) -> Iterator[concurrent.futures.Future]:
+
+		self.iterator = iter(self.iterable)
+		return self
+
+	def __next__(self) -> concurrent.futures.Future:
+
+		if self.iterator is None:
+			raise TypeError
+
+		try:
+			while len(self.futures) < self.bufsize:
+				item = next(self.iterator)
+				self.futures.add(item)
+		except StopIteration:
+			pass
+
+		done, not_done = concurrent.futures.wait(self.futures, timeout=self.timeout, return_when=concurrent.futures.FIRST_COMPLETED)
+		if done:
+			ret = done.pop()
+			self.futures = done | not_done
+			return ret
+
+		raise StopIteration
+
+	def cancel_pending(self):
+		for future in self.futures:
+			future.cancel()
+
 def _ignore_sigint():
 	# type: () -> None
 
 	""" This need to be pickle'able to work with `multiprocessing.Pool`.
 	"""
 
-	signal.signal(signal.SIGINT, signal.SIG_IGN)
+	try:
+		signal.signal(signal.SIGINT, signal.SIG_IGN)
+	except ValueError: # ignore for threadpools as 'signal only works in main thread'
+		pass
 
-def parallel_map(func, it, ordered=True, parallel=True, processes=None, bufsize=1, chunksize=1):
-	# type: (Callable[[T], U], Iterable[T], bool, bool, Optional[int], int, int) -> Iterator[U]
+def parallel_map(func: Callable[[T], U], it: Iterable[T], poolcls: Callable=None, ordered: bool=True, parallel: bool=True, workers: Optional[int]=None, bufsize: int=1, chunksize: int=1) -> Iterator[U]:
 
 	""" Parallel map which uses multiprocessing to distribute tasks.
 		`bufsize` should be used to limit memory usage when the iterable `it`
@@ -453,9 +524,12 @@ def parallel_map(func, it, ordered=True, parallel=True, processes=None, bufsize=
 
 	if parallel:
 
+		if poolcls is None:
+			poolcls = Pool
+
 		q = BoundedQueue(bufsize, it)
 
-		with Pool(processes, _ignore_sigint) as p:
+		with poolcls(workers, _ignore_sigint) as p:
 
 			if ordered:
 				process = p.imap
@@ -471,8 +545,37 @@ def parallel_map(func, it, ordered=True, parallel=True, processes=None, bufsize=
 				q.done()  # the semaphore in the bounded queue might block otherwise
 				raise
 	else:
-		for _item in it:
-			yield func(_item)
+		yield from map(func, it)
+
+def FutureWithResult(result):
+	future = concurrent.futures.Future()
+	future._result = result
+	future._state = FINISHED
+	return future
+
+def executor_map(func: Callable[[T], U], it: Iterable[T], executercls: Callable=None, ordered: bool=True, parallel: bool=True, workers: Optional[int]=None, bufsize: int=1) -> Iterator[concurrent.futures.Future]:
+
+	""" Starts processing when the iterator is started to be consumed. """
+
+	if parallel:
+
+		if executercls is None:
+			executercls = concurrent.futures.ThreadPoolExecutor
+
+		def futures() -> Iterator[concurrent.futures.Future]:
+			for item in it:
+				yield executor.submit(func, item)
+
+		with executercls(workers) as executor:
+			if ordered:
+				bufit = BufferedIterable(futures(), bufsize + executor._max_workers)
+			else:
+				bufit = CompletedFutures(futures(), bufsize + executor._max_workers)
+
+			yield from bufit
+
+	else:
+		yield from map(FutureWithResult, map(func, it))
 
 class ThreadsafeList(list):  # untested!!!
 
@@ -495,3 +598,47 @@ class ThreadsafeList(list):  # untested!!!
 
 	def __exit__(self, exc_type, exc_value, traceback):
 		self.lock.release()
+
+def idsleeprandom(i):
+	import random
+	import time
+	time.sleep(random.random()) # nosec
+	return i
+
+def benchmark():
+	""" According to these benchmark results, executor_map is faster for threads,
+		whereas parallel_map is faster for processes.
+	"""
+
+	from multiprocessing.pool import ThreadPool
+
+	from genutility.func import identity
+	from genutility.iter import consume
+	from genutility.time import PrintStatementTime
+
+	executercls = concurrent.futures.ThreadPoolExecutor
+	with PrintStatementTime("executor_map(ThreadPool) ordered=True took {delta}s"):
+		consume(executor_map(identity, range(100000), executercls=executercls, ordered=True, parallel=True))
+	with PrintStatementTime("executor_map(ThreadPool) ordered=False took {delta}s"):
+		consume(executor_map(identity, range(100000), executercls=executercls, ordered=False, parallel=True))
+
+	executercls = concurrent.futures.ProcessPoolExecutor
+	with PrintStatementTime("executor_map(ProcessPool) ordered=True took {delta}s"):
+		consume(executor_map(identity, range(10000), executercls=executercls, ordered=True, parallel=True))
+	with PrintStatementTime("executor_map(ProcessPool) ordered=False took {delta}s"):
+		consume(executor_map(identity, range(10000), executercls=executercls, ordered=False, parallel=True))
+
+	poolcls = ThreadPool
+	with PrintStatementTime("parallel_map(ThreadPool) ordered=True took {delta}s"):
+		consume(parallel_map(identity, range(100000), poolcls=poolcls, ordered=True, parallel=True))
+	with PrintStatementTime("parallel_map(ThreadPool) ordered=False took {delta}s"):
+		consume(parallel_map(identity, range(100000), poolcls=poolcls, ordered=False, parallel=True))
+
+	poolcls = Pool
+	with PrintStatementTime("parallel_map(ProcessPool) ordered=True took {delta}s"):
+		consume(parallel_map(identity, range(10000), poolcls=poolcls, ordered=True, parallel=True))
+	with PrintStatementTime("parallel_map(ProcessPool) ordered=False took {delta}s"):
+		consume(parallel_map(identity, range(10000), poolcls=poolcls, ordered=False, parallel=True))
+
+if __name__ == "__main__":
+	benchmark()
