@@ -1,12 +1,10 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
+from __future__ import generator_stop
 
 import os.path
 from configparser import ConfigParser
-from io import open
+from importlib.util import find_spec
 from typing import TYPE_CHECKING
 
-from .compat import FileNotFoundError
-from .compat.importlib.util import find_spec
 from .os import get_appdata_dir
 
 if TYPE_CHECKING:
@@ -43,10 +41,11 @@ def _load(name):
 			raise ImportError("No module named '{}'".format(name)) # or FileNotFoundError?
 
 		if spec.has_location:
+			assert spec.origin # for mypy
 			modpath = os.path.dirname(spec.origin)
 		else:
 			try:
-				modpath = spec.submodule_search_locations[0]
+				modpath = spec.submodule_search_locations[0] # type: ignore
 			except (TypeError, IndexError):
 				raise FileNotFoundError
 
