@@ -14,7 +14,7 @@ from itertools import zip_longest
 from operator import attrgetter
 from os import DirEntry, PathLike, fspath
 from pathlib import Path
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, Any, Callable, Iterable, Iterator, List, Optional, Set, Tuple, Union
 
 from .datetime import datetime_from_utc_timestamp
 from .file import FILE_IO_BUFFER_SIZE, equal_files, iterfilelike
@@ -24,9 +24,9 @@ from .os import _not_available, islink, uncabspath
 
 if TYPE_CHECKING:
 	from datetime import datetime
-	from typing import Any, Callable, Iterable, Iterator, List, Optional, Set, Tuple, Union
-	PathType = Union[str, PathLike]
-	EntryType = Union[Path, DirEntry]
+
+PathType = Union[str, PathLike]
+EntryType = Union[Path, DirEntry]
 
 logger = logging.getLogger(__name__)
 
@@ -105,7 +105,12 @@ class FileProperties(object):
 	def __iter__(self):
 		# type: () -> tuple
 
-		return self.values()
+		return iter(self.values())
+
+	def __eq__(self, other):
+		# type: (Any, ) -> bool
+
+		return self.values() == other.values()
 
 	def __repr__(self):
 		# type: () -> str
@@ -316,7 +321,7 @@ def _scandir_rec(rootentry, files=True, dirs=False, others=False, rec=True, foll
 				if files and entry.is_file(follow_symlinks=follow_symlinks):
 					yield entry
 				elif entry.is_dir(follow_symlinks=follow_symlinks):
-					if not follow_symlinks and islink(entry): # must be a windows junction
+					if not follow_symlinks and islink(entry): # must be a windows junction. entry.is_symlink() doesn't support junctions
 						continue
 					if dirs:
 						yield entry
@@ -363,6 +368,26 @@ def scandir_rec(path, files=True, dirs=False, others=False, rec=True, follow_sym
 
 		return map(modpathrelative, it)
 
+def scandir_ext(
+	path: PathType, extensions: Set[str], rec: bool=True, follow_symlinks: bool=False, relative: bool=False, errorfunc: Callable=scandir_error_log
+) -> Iterator[DirEntry]:
+
+	for entry in scandir_rec(path, files=True, dirs=False, rec=rec, follow_symlinks=follow_symlinks, relative=relative, errorfunc=errorfunc):
+		if entrysuffix(entry).lower() in extensions:
+			yield entry
+
+# fixme: benchmark and delete
+def scandir_ext_2(path: str, extensions: Set[str], rec: bool=True) -> Iterator[Path]:
+
+	if rec:
+		paths = Path(uncabspath(path)).rglob("*")
+	else:
+		paths = Path(uncabspath(path)).glob("*")
+
+	for p in paths:
+		if p.suffix.lower() in extensions:
+			yield p
+
 def _scandir_depth(rootentry, depth, errorfunc):
 
 	try:
@@ -757,7 +782,7 @@ def _scandir_counts(rootentry, files=True, others=True, rec=True, total=False, e
 		errorfunc(rootentry, e)
 
 def scandir_counts(path, files=True, others=True, rec=True, total=False, onerror=scandir_error_log):
-	# type: (PathType, bool, bool, bool, bool, Callable) -> Iterator[Tuple[DirEntry, Optional[Counts]]]
+	# type: (str, bool, bool, bool, bool, Callable) -> Iterator[Tuple[DirEntry, Optional[Counts]]]
 
 	""" A recursive variant of scandir() which also returns the number of files/directories
 		within directories.
