@@ -10,23 +10,22 @@ from operator import add, itemgetter
 from random import randrange
 from time import sleep, time
 from types import GeneratorType
-from typing import TYPE_CHECKING
+from typing import (TYPE_CHECKING, Any, Callable, Collection, Container, Dict, Generic, Iterable, Iterator, List,
+                    Optional, Sequence, Set, TextIO, Tuple, Type, TypeVar, Union)
 
 from .exceptions import EmptyIterable, IteratorExhausted
 from .ops import operator_in
 
 if TYPE_CHECKING:
-	from logging import Logger
-	from typing import (Any, Callable, Collection, Container, Dict, Iterable, Iterator, List, Optional, Sequence, Set,
-	                    TextIO, Tuple, Type, TypeVar, Union)
-
 	from .typing import Orderable, SizedIterable
-	T = TypeVar("T")
-	U = TypeVar("U")
-	V = TypeVar("V")
+
 	OrderableT = TypeVar("OrderableT", bound=Orderable)
 	ExceptionsType = Union[Type[Exception], Tuple[Type[Exception], ...]]
 
+T = TypeVar("T")
+U = TypeVar("U")
+V = TypeVar("V")
+
 logger = logging.getLogger(__name__)
 logger.addHandler(logging.NullHandler())
 
@@ -68,6 +67,8 @@ def repeatfunc(func, times=None, *args):
 	return starmap(func, repeat(args, times))
 
 def _lstr(it, length=None):
+	# type: (Iterable, Optional[int]) -> Tuple[Optional[int], str]
+
 	if length is None:
 		try:
 			length = len(it)
@@ -75,78 +76,71 @@ def _lstr(it, length=None):
 			pass
 
 	if length is not None:
-		lstr = "/{}".format(length)
+		lstr = f"/{length}"
 	else:
 		lstr = ""
 
 	return length, lstr
 
-def progressdata(it, length=None, refresh=1, end="\r", file=sys.stdout, disable=False):
-	# type: (Union[Iterable, Sequence], Optional[int], float, str, Optional[TextIO], bool) -> Iterable
+def progress(it, length=None, refresh=1, end="\r", file=sys.stdout, extra_info_callback=None, disable=False, delta=1):
+	# type: (Union[Iterable, Collection], Optional[int], float, str, Optional[TextIO], Optional[Callable[[int, Optional[int]], str]], bool, Optional[int]) -> Iterator
 
-	if disable:
-		return it
-
-	length, lstr = _lstr(it, length)
-	last = start = time()
-	total = 0
-
-	try:
-		for elm in it:
-			yield elm
-			total += len(elm)
-			current = time()
-			if current - last > refresh:
-				last = current
-				duration = current-start
-				print("Read {}{}, running for {} seconds ({:0.2e}/s).".format(total, lstr, int(duration), total/duration), end="\r", file=file)
-	except KeyboardInterrupt:
-		print("Unsafely aborted after reading {}{} in {} seconds ({:0.2e}/s).".format(total, lstr, int(last-start), total/(last-start)), end=end, file=file)
-		raise
-	else:
-		duration = last-start
-		if duration > 0:
-			print("Finished {} in {} seconds ({:0.2e}/s).".format(total, int(duration), total/duration), end=end, file=file)
-		else:
-			print("Finished {} in {} seconds.".format(total, int(duration)), end=end, file=file)
-
-def progress(it, length=None, refresh=1, end="\r", file=sys.stdout, extra_info_callback=None, disable=False):
-	# type: (Union[Iterable, Collection], Optional[int], float, str, Optional[TextIO], Optional[Callable], bool) -> Iterator
-
-	""" Wraps an iterable `it` to periodically print the progress every `refresh` seconds. """
+	""" Wraps an iterable `it` to periodically print the progress every `refresh` seconds.
+		`lengths` is the total size of `it`. `len(it)` is used to get the size if set to `None`.
+		`refresh` updates the printed output every `refresh` seconds.
+		`delta` specifies the value which is added to the counter every iteration.
+			`None` uses `len(elm)` to determine the value. Defaults to 1.
+	"""
 
 	""" todo: `from operator import length_hint`
 		Use `length_hint(it)` for a guess which might be better than nothing.
 		Can return 0.
 	"""
 
+	if delta is not None and delta < 1:
+		raise ValueError(f"`delta` must be a integer larger than 1 or `None`, not '{delta}'")
+
 	if disable:
 		return it
 
 	length, lstr = _lstr(it, length)
 	extra = ""
 	last = start = time()
-	i = 0
+	total = 0
 
 	try:
-		for i, elm in enumerate(it, 1):
+		for elm in it:
 			yield elm
+			if delta is None:
+				total += len(elm)
+			else:
+				total += delta
 			current = time()
 			if current - last > refresh:
 				last = current
-				duration = current-start
+				duration = current - start
 				if extra_info_callback:
-					extra = " [{}]".format(extra_info_callback(i, length))
-				print("{}{}, running for {} seconds ({:0.2e}/s){}.".format(i, lstr, int(duration), i/duration, extra), end="\r", file=file)
+					extra = extra_info_callback(total, length)
+					extra = f" [{extra}]"
+				print(f"{total}{lstr}, running for {int(duration)} seconds ({total/duration:0.2e}/s){extra}.", end="\r", file=file)
 	except KeyboardInterrupt:
-		print("Unsafely aborted after reading {}{} in {} seconds ({:0.2e}/s).".format(i, lstr, int(last-start), i/(last-start)), end=end, file=file)
+		duration = last - start
+		if duration > 0:
+			print(f"Unsafely aborted after reading {total}{lstr} in {int(duration)} seconds ({total/duration:0.2e}/s).", end=end, file=file)
+		else:
+			print(f"Unsafely aborted after reading {total}{lstr} in {int(duration)} seconds.", end=end, file=file)
 		raise
 	else:
-		duration = last-start
+		duration = last - start
 		if duration > 0:
-			print("Finished {} in {} seconds ({:0.2e}/s).".format(i, int(duration), i/duration), end=end, file=file)
+			print(f"Finished {total} in {int(duration)} seconds ({total/duration:0.2e}/s).", end=end, file=file)
 		else:
-			print("Finished {} in {} seconds.".format(i, int(duration)), end=end, file=file)
+			print(f"Finished {total} in {int(duration)} seconds.", end=end, file=file)
+
+def progressdata(it, length=None, refresh=1, end="\r", file=sys.stdout, extra_info_callback=None, disable=False):
+	# type: (Union[Iterable, Collection], Optional[int], float, str, Optional[TextIO], Optional[Callable[[int, Optional[int]], str]], bool) -> Iterable
+
+	return progress(it, length, refresh, end, file, extra_info_callback, disable, None)
 
 class Progress(object):
 
@@ -154,11 +148,16 @@ class Progress(object):
 		because attributes cannot be set on generator objects.
 	"""
 
-	def __init__(self, obj):
+	def __init__(self, obj, refresh=1):
+		# type: (Collection, float) -> None
+
 		self.obj = obj
+		self.refresh = refresh
 
 	def __iter__(self):
-		return progress(self.obj)
+		# type: () -> Iterator
+
+		return progress(self.obj, len(self.obj), self.refresh)
 
 	def __len__(self):
 		# type: () -> int
@@ -526,7 +525,7 @@ def list_except(it, catch=Exception):
 	return exc, ret
 
 def iter_except_ignore(iterator, exceptions, logger=logger):
-	# type: (Iterator[T], ExceptionsType, Logger) -> Iterator[T]
+	# type: (Iterator[T], ExceptionsType, logging.Logger) -> Iterator[T]
 
 	""" Ignores `exceptions` raised in `iterator`. Does not work for Generators.
 	"""
@@ -811,3 +810,36 @@ def remove_consecutive_dupes(it):
 	"""
 
 	return map(itemgetter(0), groupby(it))
+
+def nonstriter(iterable):
+	# type: (Iterable, ) -> Iterator
+
+	if isinstance(iterable, (str, bytes)):
+		raise TypeError("str or bytes")
+	else:
+		return iter(iterable)
+
+class CachedIterable(Generic[T]):
+	# https://stackoverflow.com/a/19504173
+
+	def __init__(self, iterable):
+		# type: (Iterable[T], ) -> None
+
+		self.iterable = iterable
+		self.done = False
+		self.vals = []  # type: List[T]
+
+	def __iter__(self):
+		# type: () -> Iterator[T]
+
+		if self.done:
+			return iter(self.vals)
+		return chain(self.vals, self._gen_iter())
+
+	def _gen_iter(self):
+		# type: () -> Iterator[T]
+
+		for val in self.iterable:
+			self.vals.append(val)
+			yield val
+		self.done = True
