@@ -1,47 +1,53 @@
 from __future__ import generator_stop
 
 import csv
+import datetime
 import json
 import logging
-from datetime import timedelta
 from functools import partial, wraps
 from itertools import islice
-from typing import IO, TYPE_CHECKING, Any, Callable, Dict, FrozenSet, Iterator, Optional, Sequence, Tuple, Type, Union
+from pathlib import Path
+from typing import IO, Any, Callable, Dict, FrozenSet, Iterator, Optional, Sequence, Tuple, Type, Union
 
 from typing_extensions import TypedDict
 
 from .atomic import TransactionalCreateFile, sopen
-from .datetime import now
+from .compat.datetime import datetime as compat_datetime
+from .datetime import datetime_from_utc_timestamp_ms, now
 from .file import copen
 from .filesystem import mdatetime
 from .object import args_to_key
 
-if TYPE_CHECKING:
-	from pathlib import Path
-
-	PathStr = Union[Path, str]
-
+PathStr = Union[Path, str]
 JsonDict = Dict[str, Any]
 
 logger = logging.getLogger(__name__)
 
 class BuiltinEncoder(json.JSONEncoder):
 
+	def __init__(self, *args, sort_sets=False, **kwargs):
+		# type: (*Any, bool, **Any) -> None
+
+		json.JSONEncoder.__init__(self, *args, **kwargs)
+		self.sort_sets = sort_sets
+
 	def default(self, obj):
 
 		# collections.OrderedDict is supported by default
 
 		from base64 import b85encode
-		from datetime import date, timedelta
 		from uuid import UUID
 
 		if isinstance(obj, (set, frozenset)):
-			return tuple(obj)
+			if self.sort_sets:
+				return sorted(obj)
+			else:
+				return tuple(obj)
 		elif isinstance(obj, complex):
 			return [obj.real, obj.imag]
-		elif isinstance(obj, date):
+		elif isinstance(obj, datetime.date, datetime.time, datetime.datetime):
 			return obj.isoformat()
-		elif isinstance(obj, timedelta):
+		elif isinstance(obj, datetime.timedelta):
 			return obj.total_seconds()
 		elif isinstance(obj, UUID):
 			return str(obj)
@@ -50,6 +56,32 @@ class BuiltinEncoder(json.JSONEncoder):
 
 		return json.JSONEncoder.default(self, obj)
 
+class BuiltinRoundtripEncoder(json.JSONEncoder):
+
+	def default(self, obj):
+		if isinstance(obj, datetime.timedelta):
+			return {"$timedelta": (obj.days, obj.seconds, obj.microseconds)}
+		elif isinstance(obj, datetime.datetime):
+			return {"$datetime": obj.isoformat()}
+
+		return json.JSONEncoder.default(self, obj)
+
+class BuiltinRoundtripDecoder(json.JSONDecoder):
+	def __init__(self, *args, **kwargs):
+		json.JSONDecoder.__init__(self, object_hook=self.object_hook, *args, **kwargs)
+
+	def object_hook(self, obj):
+		if len(obj) == 1:
+			key, value = next(iter(obj.items()))
+			if key == "$timedelta":
+				return datetime.timedelta(*value)
+			elif key == "$datetime":
+				return compat_datetime.fromisoformat(value)
+			elif key == "$date":  # for compatibility with bson.json_util
+				return datetime_from_utc_timestamp_ms(value)
+
+		return obj
+
 def read_json_schema(path):
 	# type: (PathStr, ) -> JsonDict
 
@@ -79,8 +111,8 @@ def read_json(path, schema=None, cls=None, object_hook=None):
 	validate(obj, schema)
 	return obj
 
-def write_json(obj, path, schema=None, ensure_ascii=False, cls=None, indent=None, sort_keys=False, default=None, safe=False):
-	# type: (Any, PathStr, Optional[Union[str, JsonDict]], bool, Optional[Type[json.JSONEncoder]], Optional[Union[str, int]], bool, Optional[Callable], bool) -> None
+def write_json(obj, path, schema=None, ensure_ascii=False, cls=None, indent=None, sort_keys=False, default=None, safe=False, **kw):
+	# type: (Any, PathStr, Optional[Union[str, JsonDict]], bool, Optional[Type[json.JSONEncoder]], Optional[Union[str, int]], bool, Optional[Callable], bool, **Any) -> None
 
 	""" Writes python object `obj` to `path` as json files and optionally validates the object
 		according to `schema`. The validation requires `jsonschema`.
@@ -97,7 +129,7 @@ def write_json(obj, path, schema=None, ensure_ascii=False, cls=None, indent=None
 		validate(obj, schema)
 
 	with sopen(path, "wt", encoding="utf-8", safe=safe) as fw:
-		json.dump(obj, fw, ensure_ascii=ensure_ascii, cls=cls, indent=indent, sort_keys=sort_keys, default=default)
+		json.dump(obj, fw, ensure_ascii=ensure_ascii, cls=cls, indent=indent, sort_keys=sort_keys, default=default, **kw)
 
 class JsonLoadKwargs(TypedDict):
 	cls: Optional[Type[json.JSONDecoder]]
@@ -150,6 +182,18 @@ class json_lines(object):
 			Use text modes instead. Binary read-only is fine.
 		"""
 
+		if isinstance(file, str):
+			if file.startswith("s3://"):
+				message = f"""Reading data from S3 is not supported natively. Use `smart-open` package:
+```python
+from smart_open import open
+with open("{file}", "r") as fr:
+	with json_lines.from_stream(fr) as jl:
+		for doc in jl:
+			pass
+```"""
+				raise ValueError(message)
+
 		if set(mode) not in (set("rt"), set("rb"), set("xt"), set("xb"), set("wt"), set("at"), set("r+t"), set("w+t")):
 			raise ValueError(f"mode cannot be `{mode}`")
 
@@ -173,10 +217,9 @@ class json_lines(object):
 	def __exit__(self, exc_type, exc_value, traceback):
 		self.close()
 
-	def iterrange(self, start=0, stop=None):
-		# type: (int, Optional[int]) -> Iterator
+	def iterrange(self, start: int=0, stop: Optional[int]=None) -> Iterator:
 
-		linenum = start+1
+		linenum = start + 1
 		try:
 			for line in islice(self.f, start, stop):
 				line = line.rstrip().lstrip("\x00") # fixme: strip \0 is only a temp fix!
@@ -189,8 +232,7 @@ class json_lines(object):
 			logger.error("JSON Lines parse error in line %s: '%r'", linenum, line)
 			raise
 
-	def __iter__(self):
-		# type: () -> Iterator
+	def __iter__(self) -> Iterator:
 
 		return self.iterrange()
 
@@ -220,8 +262,7 @@ def read_json_lines(file, object_hook=None):
 		for obj in fr:
 			yield obj
 
-def jl_to_csv(jlpath, csvpath, keyfunc, mode="xt"):
-	# type: (PathStr, str, Callable[[JsonDict], Sequence[str]], str) -> None
+def jl_to_csv(jlpath: PathStr, csvpath: str, keyfunc: Callable[[JsonDict], Sequence[str]], mode: str="xt") -> None:
 
 	with json_lines.from_path(jlpath, "rt") as fr:
 		with open(csvpath, mode, encoding="utf-8", newline="") as csvfile:
@@ -237,13 +278,16 @@ def key_to_hash(key, default=None):
 	return md5(binary).hexdigest()  # nosec
 
 def cache(path, duration=None, ensure_ascii=False, indent=None, sort_keys=False, default=None, object_hook=None):
-	# type: (Path, Optional[timedelta], bool, Optional[Union[str, int]], bool, Optional[Callable], Optional[Callable]) -> Callable
+	# type: (Path, Optional[datetime.timedelta], bool, Optional[Union[str, int]], bool, Optional[Callable], Optional[Callable]) -> Callable
 
 	""" Decorator to cache results of function to json files at `path` for `duration`.
 		The remaining parameters are passed through to `json.dump`.
 	"""
 
-	duration = duration or timedelta.max
+	if duration is None:
+		_duration = datetime.timedelta.max
+	else:
+		_duration = duration
 
 	def decorator(func):
 		# type: (Callable, ) -> Callable
@@ -255,7 +299,7 @@ def cache(path, duration=None, ensure_ascii=False, indent=None, sort_keys=False,
 			fullpath = path / hash
 
 			try:
-				invalid = now() - mdatetime(fullpath) > duration
+				invalid = now() - mdatetime(fullpath) > _duration
 			except FileNotFoundError:
 				invalid = True
 
@@ -271,13 +315,13 @@ def cache(path, duration=None, ensure_ascii=False, indent=None, sort_keys=False,
 	return decorator
 
 def jsonlines_cache(path, duration=None, ensure_ascii=False, sort_keys=False, default=None, object_hook=None):
-	# type: (Path, Optional[timedelta], bool, bool, Optional[Callable], Optional[Callable]) -> Callable
+	# type: (Path, Optional[datetime.timedelta], bool, bool, Optional[Callable], Optional[Callable]) -> Callable
 
 	""" Decorator to cache results of function to jsonlines files at `path` for `duration`.
 		The remaining parameters are passed through to `json_lines.from_path`.
 	"""
 
-	duration = duration or timedelta.max
+	duration = duration or datetime.timedelta.max
 
 	def decorator(func):
 		# type: (Callable, ) -> Callable
