@@ -1,18 +1,16 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
+from __future__ import generator_stop
 
+from os import PathLike, fspath
 from typing import TYPE_CHECKING
 
 from pymediainfo import MediaInfo
 
-from .compat.os import PathLike, fspath
-
 if TYPE_CHECKING:
+	from pathlib import Path
 	from typing import Union
 
 	from pymediainfo import Track
 
-	from .compat.pathlib import Path
-
 class MediaInfoHelper(object):
 
 	def __init__(self, path):
