@@ -78,8 +78,8 @@ def key_to_hash(key, protocol=None):
 	binary = pickle.dumps(key, protocol=protocol)
 	return md5(binary).hexdigest()  # nosec
 
-def cache(path, duration=None, generator=False, protocol=HIGHEST_PROTOCOL, ignoreargs=False, verbose=False):
-	# type: (Path, Optional[timedelta], bool, int, bool, bool) -> Callable[[Callable], Callable]
+def cache(path, duration=None, generator=False, protocol=HIGHEST_PROTOCOL, ignoreargs=False, verbose=False, keyfunc=None, consume=False, return_cached=False):
+	# type: (Path, Optional[timedelta], bool, int, bool, bool, Optional[Callable[[tuple, dict], str]], bool, bool) -> Callable[[Callable], Callable]
 
 	""" Decorator to cache function calls. Doesn't take function arguments into regard.
 		It's using `pickle` to deserialize the data. So don't use it with untrusted inputs.
@@ -95,7 +95,13 @@ def cache(path, duration=None, generator=False, protocol=HIGHEST_PROTOCOL, ignor
 			where `ppv` is assigned the  pickle protocol version.
 	"""
 
-	_duration = duration or timedelta.max
+	if not generator and consume:
+		raise ValueError("consume can onlt be used for generator")
+
+	if duration is None:
+		_duration = timedelta.max
+	else:
+		_duration = duration
 
 	if not verbose:
 		context = nullcontext
@@ -110,7 +116,10 @@ def cache(path, duration=None, generator=False, protocol=HIGHEST_PROTOCOL, ignor
 			# type: (*Any, **Any) -> Any
 
 			if not ignoreargs:
-				hash = key_to_hash(args_to_key(args, kwargs), protocol=protocol)
+				if keyfunc is None:
+					hash = key_to_hash(args_to_key(args, kwargs), protocol=protocol)
+				else:
+					hash = keyfunc(args, kwargs)
 				fullpath = path / hash
 			else:
 				if args or kwargs:
@@ -123,23 +132,35 @@ def cache(path, duration=None, generator=False, protocol=HIGHEST_PROTOCOL, ignor
 				invalid = True
 
 			if invalid:
+				cached = False
 				if not ignoreargs:
 					path.mkdir(parents=True, exist_ok=True)
 
-				if generator:
+				if generator and not consume:
 					it = func(*args, **kwargs)
-					return write_iter(it, fullpath, protocol=protocol, safe=True)
+					logger.info("Writing iterable to cache: %s", fullpath)
+					result = write_iter(it, fullpath, protocol=protocol, safe=True)
 				else:
 					with context("Result calculated in {delta} seconds"):
-						result = func(*args, **kwargs)
+						if generator and consume:
+							result = list(func(*args, **kwargs))
+						else:
+							result = func(*args, **kwargs)
 					write_pickle(result, fullpath, protocol=protocol, safe=True)
-					return result
 			else:
-				if generator:
-					return read_iter(fullpath)
+				cached = True
+				if generator and not consume:
+					logger.info("Loading iterable from cache: %s", fullpath)
+					result = read_iter(fullpath)
 				else:
 					with context("Result loaded from cache in {delta} seconds"):
-						return read_pickle(fullpath)
+						result = read_pickle(fullpath)
+
+			if return_cached:
+				return cached, result
+			else:
+				return result
+
 		return inner
 
 	return decorator
