@@ -1,23 +1,13 @@
+from __future__ import generator_stop
+
 import asyncio
 import csv
 import logging
 import re
 import time
 from collections.abc import Mapping, Sequence
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Coroutine,
-    Dict,
-    Generic,
-    Iterable,
-    Iterator,
-    List,
-    Optional,
-    Tuple,
-    TypeVar,
-    Union,
-)
+from typing import (TYPE_CHECKING, Any, Coroutine, Dict, Generic, Iterable, Iterator, List, Optional, Tuple, TypeVar,
+                    Union)
 
 import aiohttp
 import requests
@@ -400,10 +390,11 @@ class LiveAgentBase(Generic[ReturnTGet, ReturnTPost]):
 
 		return prechat_details
 
-	@staticmethod
-	def _availibility(res):
+	def _availibility(self, res):
 		# type: (JsonDict, ) -> bool
 
+		# this is not a staticmethod as to include potentially useful stack-info in logs
+
 		for msg in res["messages"]:
 			if msg["type"] == "Availability":
 				return msg["message"]["results"][0].get("isAvailable", False)
@@ -469,7 +460,7 @@ class LiveAgentBase(Generic[ReturnTGet, ReturnTPost]):
 	def rest_chasitor_init(self, key, affinity_token, session_id, visitor_name, user_agent="", language="en-US",
 		screen_resolution="1920x1080", prechat_details=None, prechat_entities=None,
 		receive_queue_updates=True, is_post=True):
-		# type: (str, str, str, str, str, str, str, List[JsonDict], List[JsonDict], bool, bool) -> ReturnTPost
+		# type: (str, str, str, str, str, str, str, Optional[List[JsonDict]], Optional[List[JsonDict]], bool, bool) -> ReturnTPost
 
 		endpoint = "/chat/rest/Chasitor/ChasitorInit"
 
@@ -639,7 +630,7 @@ class LiveAgent(LiveAgentBase[JsonDict, bytes]):
 	# high level
 
 	def connect(self, visitor_name, slots=None, prechat_details=None, prechat_entities=None):
-		# type: (str, Optional[Union[Dict[str, str], List[JsonDict]]], List[JsonDict], List[JsonDict]) -> None
+		# type: (str, Optional[Union[Dict[str, str], List[JsonDict]]], Optional[List[JsonDict]], Optional[List[JsonDict]]) -> None
 
 		""" Connect using `visitor_name` as name.
 		"""
@@ -745,7 +736,7 @@ class LiveAgentAsync(LiveAgentBase[Coroutine[Any, Any, JsonDict], Coroutine[Any,
 	# high level
 
 	async def connect(self, visitor_name, slots=None, prechat_details=None, prechat_entities=None):
-		# type: (str, Optional[Union[Dict[str, str], List[JsonDict]]], List[JsonDict], List[JsonDict]) -> None
+		# type: (str, Optional[Union[Dict[str, str], List[JsonDict]]], Optional[List[JsonDict]], Optional[List[JsonDict]]) -> None
 
 		""" Connect using `visitor_name` as name.
 		"""
