@@ -1,19 +1,16 @@
 from __future__ import generator_stop
 
 import logging
+import os
 import sys
-from time import sleep
-from typing import TYPE_CHECKING
-
-if TYPE_CHECKING:
-	from typing import IO, Any, Optional
-
 from shutil import get_terminal_size
+from time import sleep
+from typing import IO, Any, Callable, Optional, Type, TypeVar, Union
 
+T = TypeVar("T")
 _terminal_width = get_terminal_size((80, 30)).columns
 
-def print_line(char="-", length=_terminal_width, file=None, flush=False):
-	# type: (str, int, IO, bool) -> None
+def print_line(char: str="-", length: int=_terminal_width, file: Optional[IO]=None, flush: bool=False) -> None:
 	""" file=None, defaults to sys.stdout in print(). print() handles sys.stdout==None fine. """
 
 	print(char*length, end="", file=file, flush=flush)
@@ -41,8 +38,7 @@ class PrintOnError(object):
 		if exc_type:
 			print(*self.args, **self.kwargs)
 
-def safe_input(s, block=10):
-	# type: (str, int) -> str
+def safe_input(s: str, block: int=10) -> str:
 
 	""" Makes sure ctrl-c on input() only raises KeyboardInterrupt and not EOFError+KeyboardInterrupt.
 		Waits at most `block` seconds to be sure.
@@ -54,8 +50,7 @@ def safe_input(s, block=10):
 		sleep(block) # can be interrupted by KeyboardInterrupt
 		raise
 
-def info_print(msg=None, args=tuple(), exception=None):
-	# type: (Optional[str], tuple, Exception) -> None
+def info_print(msg: Optional[str]=None, args: tuple=tuple(), exception: Optional[Exception]=None) -> None:
 
 	# not (msg or exception) this doesn't do anything
 
@@ -68,15 +63,16 @@ def info_print(msg=None, args=tuple(), exception=None):
 		if msg:
 			print(msg % args)
 
-def input_ex(s, file_out=sys.stdout, file_in=sys.stdin):
-	# type: (str, IO, IO) -> str
+def input_ex(s: str, file_out: IO=sys.stdout, file_in: IO=sys.stdin) -> str:
 
 	print(s, file=file_out, end="")
 	file_out.flush()
 	return sys.stdin.readline().rstrip("\n")
 
 # was: raw_input_ex
-def input_type(s, type=None, predicate=None, errormsg=None, exception=Exception, file=sys.stdout):
+def input_type(
+	s: str, type: Optional[Callable[[str], T]]=None, predicate: Optional[Callable[[str], bool]]=None, errormsg: Optional[str]=None, exception: Type[Exception]=Exception, file: IO=sys.stdout
+) -> Union[str, T]:
 	while True:
 		ret = input_ex(s, file_out=file)
 		if predicate and not predicate(ret):
@@ -92,8 +88,7 @@ def input_type(s, type=None, predicate=None, errormsg=None, exception=Exception,
 				continue
 		return ret
 
-def confirm(msg, yesno=True):
-	# type: (str, bool) -> bool
+def confirm(msg: str, yesno: bool=True) -> bool:
 
 	if yesno:
 		s = input_type(msg+" (yes, no): ", predicate=lambda x: x.strip().lower() in {"yes", "y", "n", "no"})
@@ -101,15 +96,21 @@ def confirm(msg, yesno=True):
 	else:
 		return input_type(msg+" (anything, nothing): ", type=bool) # bool cannot throw if given string I think...
 
-def waitquit(msg=None, args=tuple(), exception=None): # fixme: ferutility.printing breaks this
+def waitquit(msg=None, args=tuple(), exception: Optional[Exception]=None): # fixme: ferutility.printing breaks this
 	info_print(msg, args, exception)
 	input("Press enter to exit...")
 	sys.exit(msg or exception)
 
-def waitcontinue(msg=None, args=tuple(), exception=None):
+def waitcontinue(msg=None, args=tuple(), exception: Optional[Exception]=None):
 	info_print(msg, args, exception)
 	input("Press enter to continue...")
 
-def errorquit(msg=None, args=tuple(), exception=None):
+def errorquit(msg=None, args=tuple(), exception: Optional[Exception]=None):
 	info_print(msg, args, exception)
 	sys.exit(msg or exception)
+
+def print_terminal_progress_line(out: str, file=None) -> None:
+	columns, lines = os.get_terminal_size()
+	if len(out) >= columns:
+		out = out[: columns - 4] + "..."
+	print(out, end="\r", file=file)
