@@ -1,6 +1,4 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-from builtins import chr, map, range
+from __future__ import generator_stop
 
 from collections import defaultdict
 from pathlib import Path
